import logging

import arango
import arango.exceptions
from arango.database import StandardDatabase
from arango import ArangoClient

from dogesec_commons.objects import conf

logging.basicConfig(
    level=logging.INFO,
    format='[ARANGODB VIEW] %(levelname)s %(asctime)s %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S'
)

from django.conf import settings


SORT_FIELDS = [
    "id",
    "type",
    "created",
    "modified",
    "name",
]

FILTER_HIDDEN_FIELDS = [
    "_stix2arango_note",
    "_id",
    "_from",
    "_to",
    "_is_ref",
    "_arango_cti_processor_note",
    "_arango_cve_processor_note",
]

FILTER_FIELDS_VERTEX = [
    "type",
    "name",
    "labels",
]
FILTER_FIELDS_EDGE = [
    "source_ref",
    "target_ref",
    "relationship_type",
]
FILTER_SCO_FIELDS = ['value', 'path', 'subject', 'number', 'pid', 'string', 'key', 'iban_number', 'payload_bin', 'hash', 'display_name', 'protocols', 'name', 'body']
FILTER_FIELDS = list(set(FILTER_FIELDS_EDGE + FILTER_FIELDS_VERTEX))


def create_database(client: ArangoClient, sys_db: StandardDatabase, db_name):
    logging.info(f"creating database {db_name}")
    try:
        sys_db.create_database(db_name)
    except arango.exceptions.DatabaseCreateError as e:
        logging.error(e)
    return client.db(
        name=db_name, username=settings.ARANGODB_USERNAME, password=settings.ARANGODB_PASSWORD, verify=True
    )


def create_view(db: StandardDatabase, view_name, sort_fields=SORT_FIELDS, filter_fields=[SORT_FIELDS, FILTER_FIELDS_VERTEX, FILTER_FIELDS_EDGE, FILTER_HIDDEN_FIELDS, FILTER_SCO_FIELDS]):
    logging.info(f"creating view {view_name} in {db.name}")
    primary_sort = []
    for field in sort_fields:
        primary_sort.append(dict(field=field, direction="asc"))
        primary_sort.append(dict(field=field, direction="desc"))

    all_fields = [*filter_fields, sort_fields]

    try:
        logging.info("try updating view (%s) if exists", view_name)
        return update_view(db, all_fields, view_name)
    except BaseException as e:
        logging.info(f"view update not possible: {e}")

    try:
        logging.info("create new view: %s", view_name)
        return db.create_arangosearch_view(
            view_name, {"primarySort": primary_sort, "storedValues": all_fields}
        )
    except arango.exceptions.ViewCreateError as e:
        logging.error(e)
    return db.view(view_name)


def get_link_properties(collection_name: str):
    if collection_name.endswith("_vertex_collection"):
        return {
            "fields": {name: {} for name in FILTER_FIELDS_VERTEX},
        }
    elif collection_name.endswith("_edge_collection"):
        return {
            "fields": {name: {} for name in FILTER_FIELDS_EDGE},
        }
    else:
        return None


def link_one_collection(db: StandardDatabase, view_name, collection_name):
    logging.info(f"linking collection {collection_name} to {view_name}")
    view = db.view(view_name)
    link = {"includeAllFields": True}
    if link and collection_name:
        view["links"][collection_name] = link
    db.update_arangosearch_view(view_name, view)
    logging.info(f"linked collection {collection_name} to {view_name}")


def update_view(db: StandardDatabase, filter_fields, view_name) -> bool:
    view = db.view(view_name)
    view_fields = [sv['fields'] for sv in view['stored_values']]
    def hash_fields(fields: list[list[str]]) -> set[str]:
        hashes = set()
        for ff in fields:
            hash_val = 0
            for f in set(ff):
                hash_val += hash(f)
            hashes.add(hash_val)
        return hashes
    view_hash = hash_fields(view_fields)
    fields_hash = hash_fields(filter_fields)
    
    if view_hash == fields_hash:
        return view
    
    logging.info("old hash does not match new hash, updating...")

    view['storedValues'] = [{"fields": v} for v in filter_fields]
    view["stored_values"] = filter_fields

    new_view = db.update_arangosearch_view(view_name, view)

    # should update but for some reason update not working, so recreate instead
    new_view_hash = hash_fields([sv['fields'] for sv in new_view['stored_values']])
    if new_view_hash != fields_hash:
        db.delete_view(view_name, ignore_missing=True)
        raise Exception("recreate because update is impossible")
    return new_view

def link_all_collections(db: StandardDatabase, view: dict):
    links = view.get("links", {})
    view_name = view["name"]
    for collection in db.collections():
        collection_name = collection["name"]
        if collection["system"]:
            continue
        links[collection_name] = {"includeAllFields": True}
        if not links[collection_name]:
            del links[collection]
            continue
        logging.info(f"linking collection {collection_name} to {view_name}")
    db.update_arangosearch_view(view_name, view)
    logging.info(f"linked {len(links)} collections to view")


def startup_func():
    logging.info("setting up database")
    client = ArangoClient(settings.ARANGODB_HOST_URL)
    sys_db = client.db(username=settings.ARANGODB_USERNAME, password=settings.ARANGODB_PASSWORD)
    db = create_database(client, sys_db, conf.DB_NAME)
    view = create_view(db, conf.ARANGODB_DATABASE_VIEW)
    link_all_collections(db, view)
    logging.info("app ready")


from rest_framework.filters import BaseFilterBackend
from datetime import datetime, UTC
from django.forms import DateTimeField
from django_filters.rest_framework import filters


class DatetimeFieldUTC(DateTimeField):
    def to_python(self, value):
        value = super().to_python(value)
        return value and value.astimezone(UTC)


class DatetimeFilter(filters.Filter):
    field_class = DatetimeFieldUTC


class MinMaxDateFilter(BaseFilterBackend):
    min_val = datetime.min
    max_value = datetime.max

    def get_fields(self, view):
        out = {}
        fields = getattr(view, "minmax_date_fields", [])
        if not isinstance(fields, list):
            return out
        for field in fields:
            out[f"{field}_max"] = field
            out[f"{field}_min"] = field
        return out

    def parse_date(self, value):
        return DatetimeFieldUTC().to_python(value)

    def filter_queryset(self, request, queryset, view):
        valid_fields = self.get_fields(view)
        valid_params = [
            (k, v) for k, v in request.query_params.items() if k in valid_fields
        ]
        queries = {}
        for param, value in valid_params:
            field_name = valid_fields[param]
            if param.endswith("_max"):
                queries[f"{field_name}__lte"] = self.parse_date(value)
            else:
                queries[f"{field_name}__gte"] = self.parse_date(value)
        return queryset.filter(**queries)

    def get_schema_operation_parameters(self, view):
        parameters = []
        valid_fields = self.get_fields(view)
        for query_name, field_name in valid_fields.items():
            _type = "Maximum"
            if query_name.endswith("min"):
                _type = "Minimum"
            parameter = {
                "name": query_name,
                "required": False,
                "in": "query",
                "description": f"{_type} value of `{field_name}` to filter by in format `YYYY-MM-DD`.",
                "schema": {
                    "type": "string",
                    "format": "date",
                },
            }
            parameters.append(parameter)
        return parameters



import time
from django.conf import settings
from django.http import HttpRequest
import rest_framework.request
from dogesec_commons.objects.db_view_creator import startup_func
from dogesec_commons.objects.helpers import ArangoDBHelper
from stix2arango.stix2arango import Stix2Arango
import contextlib
from arango.client import ArangoClient


def make_s2a_uploads(collection, objects):
    helper = ArangoDBHelper()
    helper.db.collection(collection).insert_many(objects)
    return helper


def as_arango2stix_db(db_name):
    if db_name.endswith("_database"):
        return "_".join(db_name.split("_")[:-1])
    return db_name


@contextlib.contextmanager
def make_s2a_uploads(
    uploads: list[tuple[str, list[dict]]],
    truncate_collection=False,
    database=settings.ARANGODB_DATABASE,
    **kwargs,
):
    database = as_arango2stix_db(database)

    for collection, objects in uploads:
        s2a = Stix2Arango(
            database=database,
            collection=collection,
            file="",
            host_url=settings.ARANGODB_HOST_URL,
            **kwargs,
        )
        s2a.run(data=dict(type="bundle", id="", objects=objects))

    startup_func()
    time.sleep(1)
    yield s2a

    if truncate_collection:
        for collection, _ in uploads:
            s2a.arango.db.collection(collection + "_vertex_collection").truncate()
            s2a.arango.db.collection(collection + "_edge_collection").truncate()


@contextlib.contextmanager
def make_uploads(collection_name):
    client = ArangoClient(hosts=settings.ARANGODB_HOST_URL)
    db = client.db(
        settings.ARANGODB_DATABASE,
        username=settings.ARANGODB_USERNAME,
        password=settings.ARANGODB_PASSWORD,
    )
    transaction_db = db.begin_transaction(write=collection_name, allow_implicit=True)
    try:
        if not transaction_db.has_collection(collection_name):
            transaction_db.create_collection(collection_name)
        yield transaction_db
    except:
        raise
    finally:
        transaction_db.abort_transaction()


def request_from_queries(**queries):
    r = rest_framework.request.Request(HttpRequest())
    r.query_params.update(queries)
    return r



from rest_framework.views import exception_handler
from rest_framework.exceptions import ValidationError, PermissionDenied
from django.core import exceptions as django_exceptions
from django.http import JsonResponse, Http404
import rest_framework.exceptions
import django.db.models.deletion


def custom_exception_handler(exc, context):
    if isinstance(exc, django_exceptions.ValidationError):
        exc = ValidationError(detail=exc.messages, code=exc.code)
    
    if isinstance(exc, django.db.models.deletion.ProtectedError):
        return JsonResponse({'code': 403, 'message': "cannot delete object(s) because they are referenced through protected foreign keys.", 'details': {'protected_objects': [str(f) for f in exc.protected_objects]}}, status=403)
        
    resp = exception_handler(exc, context)
    if resp is not None:
        if isinstance(resp.data, dict) and 'detail' in resp.data:
                resp.data = resp.data['detail']
        if isinstance(resp.data, str):
            resp.data = dict(code=resp.status_code, message=resp.data)
        if isinstance(resp.data, list):
            resp.data = dict(code=resp.status_code, details={'detail':resp.data})
        else:
            resp.data = dict(code=resp.status_code, details=resp.data)
        resp.data.setdefault('message', resp.status_text)
        resp = JsonResponse(data=resp.data, status=resp.status_code)
    return resp


from .models import Profile
from drf_spectacular.utils import extend_schema, extend_schema_view, OpenApiParameter
from drf_spectacular.types import OpenApiTypes
from ..utils import Pagination, Ordering

from rest_framework import viewsets, response, mixins, exceptions
from django_filters.rest_framework import (
    DjangoFilterBackend,
    FilterSet,
    Filter,
    BooleanFilter,
)
from .serializers import (
    DEFAULT_400_ERROR,
    DEFAULT_404_ERROR,
    Txt2stixExtractorSerializer,
)
from django.forms import NullBooleanField

from .serializers import ProfileSerializer

from drf_spectacular.utils import extend_schema, extend_schema_view
import django.db.models.deletion
import textwrap

EXTRACTOR_TYPES = ["lookup", "pattern", "ai"]


@extend_schema_view(
    list=extend_schema(
        summary="Search profiles",
        description=textwrap.dedent(
            """
            Profiles determine how txt2stix processes the text in each File. A profile consists of extractors. You can search for existing profiles here.
            """
        ),
        responses={400: DEFAULT_400_ERROR, 200: ProfileSerializer},
    ),
    retrieve=extend_schema(
        summary="Get a profile",
        description=textwrap.dedent(
            """
            View the configuration of an existing profile. Note, existing profiles cannot be modified.
            """
        ),
        responses={
            400: DEFAULT_400_ERROR,
            404: DEFAULT_404_ERROR,
            200: ProfileSerializer,
        },
    ),
    create=extend_schema(
        summary="Create a new profile",
        description=textwrap.dedent(
            """
            Add a new Profile that can be applied to new Files. A profile consists of extractors. You can find available extractors via their respective endpoints.

            The following key/values are accepted in the body of the request:

            * `name` (required - must be unique)
            * `identity_id` (optional): a STIX Identity ID that you want to use to create the profile. Pass the full identity ID, e.g. `identity--de9fb9bd-7895-4b23-aa03-49250d9263c9`
            * `ai_content_check_provider` (optional, AI provider:model): Setting a value will get the AI to try and classify the text in the input to 1) determine if it is talking about threat intelligence, and 2) what type of threat intelligence it is talking about. You pass `provider:model` (e.g. `"openai:gpt-4o"`) with this flag to determine the AI model you wish to use to perform the check. 
            * `ai_extract_if_no_incidence` (optional, boolean): if content check decides the report is not related to cyber security intelligence (e.g. vendor marketing), then you can use this setting to decide wether or not script should proceed. Setting to `false` will stop processing at extraction if the AI determines if the report entered is not related to security intelligence. It is designed to save AI tokens processing unknown content at scale in an automated way. Will only work if `ai_content_check_provider` set (uses the same AI model).
            * `extract_text_from_image` (required - boolean): whether to convert the images found in a blog to text. Requires a Google Vision key to be set. This is a [file2txt](https://github.com/muchdogesec/file2txt) setting.
            * `defang` (required - boolean): whether to defang the observables in the blog. e.g. turns `1.1.1[.]1` to `1.1.1.1` for extraction. This is a [file2txt](https://github.com/muchdogesec/file2txt) setting.
            * `ignore_image_refs` (optional, default `true`): whether to ignore embedded image references. This is a [txt2stix](https://github.com/muchdogesec/txt2stix/) setting.
            * `ignore_link_refs` (optional, default `true`): whether to ignore embedded link references. This is a [txt2stix](https://github.com/muchdogesec/txt2stix/) setting.
            * `extractions` (required - at least one extraction ID): can be obtained from the GET Extractors endpoint. This is a [txt2stix](https://github.com/muchdogesec/txt2stix/) setting.
            * `ai_settings_extractions` (required if AI extraction mode used, AI provider:model): A list of AI providers and models to be used for extraction in format `["provider:model","provider:model"]` e.g. `["openai:gpt-4o"]`. This is a [txt2stix](https://github.com/muchdogesec/txt2stix/) setting.
            * `ignore_extraction_boundary` (optional, default `false`): defines if a string boundary can generate multiple extractions (e.g. `url`, `domain`, etc). Setting to `true` will allow multiple extractions from the same string. This is a [txt2stix](https://github.com/muchdogesec/file2txt) setting.
            * `relationship_mode` (required): either `ai` or `standard`. Required AI provider to be configured if using `ai` mode. This is a [txt2stix](https://github.com/muchdogesec/txt2stix/) setting.
            * `ai_settings_relationships` (required if AI relationship used, AI provider:model): An AI provider and models to be used for relationship generation in format `"provider:model"` e.g. `"openai:gpt-4o"`. This is a [txt2stix](https://github.com/muchdogesec/txt2stix/) setting.
            * `generate_pdf` (optional, boolean): default is `false`. Will generate a PDF of the input if set to true 
            * `ai_create_attack_flow` (optional, boolean): default is `false`. Passing as `true` will prompt the AI model (the same entered for `ai_settings_relationships`) to generate an [Attack Flow](https://center-for-threat-informed-defense.github.io/attack-flow/) for the MITRE ATT&CK extractions to define the logical order in which they are being described. You must pass `--ai_settings_relationships` for this to work (uses same AI model). This will only work if at least one MITRE ATT&CK extraction is enabled (and it extracts data).
            * `ai_create_attack_navigator_layer` (optional, boolean): default is `false`. Passing this as `true` will generate MITRE ATT&CK Navigator layers for MITRE ATT&CK extractions. You must pass `--ai_settings_relationships` for this to work (uses same AI model). This will only work if at least one MITRE ATT&CK extraction is enabled (and it extracts data).
            * `ignore_embedded_relationships` (optional, default: false): boolean, if `true` passed, this will stop ANY embedded relationships from being generated. This applies for all object types (SDO, SCO, SRO, SMO). If you want to target certain object types see `ignore_embedded_relationships_sro` and `ignore_embedded_relationships_sro` flags. This is a [stix2arango](https://github.com/muchdogesec/stix2arango) setting.
            * `ignore_embedded_relationships_sro` (optional, default: false): boolean, if `true` passed, will stop any embedded relationships from being generated from SRO objects (`type` = `relationship`). This is a [stix2arango](https://github.com/muchdogesec/stix2arango) setting.
            * `ignore_embedded_relationships_smo` (optional, default: false): boolean, if `true` passed, will stop any embedded relationships from being generated from SMO objects (`type` = `marking-definition`, `extension-definition`, `language-content`). This is a [stix2arango](https://github.com/muchdogesec/stix2arango) setting.
            * `include_embedded_relationships_attributes` (optional, default: n/a): stix `_ref` or `_refs` attribute. If you only want to create embedded relationships from certain keys (attributes) in a STIX object you can pass these attributes here. e.g. `object_refs` . In this example, embedded relationships to all objects listed in `object_refs` attributes will be created between source (the objects that house these attibutes) and destinations (the objects listed as values for these attributes

            A profile `id` is generated using a UUIDv5. The namespace used is is set using the `STIXIFIER_NAMESPACE` in dogesec tools, and the `name+identity_id` is used as the value (e.g a namespace of `9779a2db-f98c-5f4b-8d08-8ee04e02dbb5` and value `my profile+identity--de9fb9bd-7895-4b23-aa03-49250d9263c9` would have the `id`: `05004944-0eff-507e-8ef8-9ebdd043a51b`). Note, the name

            You cannot modify a profile once it is created. If you need to make changes, you should create another profile with the changes made. If it is essential that the same `name` + `identity_id` value be used, then you must first delete the profile in order to recreate it.
            """
        ),
        responses={400: DEFAULT_400_ERROR, 201: ProfileSerializer},
    ),
    destroy=extend_schema(
        summary="Delete a profile",
        description=textwrap.dedent(
            """
            Delete an existing profile.

            Note: it is not currently possible to delete a profile that is referenced in an existing object. You must delete the objects linked to the profile first.
            """
        ),
        responses={404: DEFAULT_404_ERROR, 204: None},
    ),
)
class ProfileView(viewsets.ModelViewSet):
    openapi_tags = ["Profiles"]
    serializer_class = ProfileSerializer
    http_method_names = ["get", "post", "delete"]
    pagination_class = Pagination("profiles")
    lookup_url_kwarg = "profile_id"
    openapi_path_params = [
        OpenApiParameter(
            lookup_url_kwarg,
            location=OpenApiParameter.PATH,
            type=OpenApiTypes.UUID,
            description="The `id` of the Profile.",
        )
    ]

    ordering_fields = ["name", "created"]
    ordering = "created_descending"
    filter_backends = [DjangoFilterBackend, Ordering]

    class filterset_class(FilterSet):
        name = Filter(
            help_text="Searches Profiles by their `name`. Search is wildcard. For example, `ip` will return Profiles with names `ip-extractions`, `ips`, etc.",
            lookup_expr="icontains",
        )
        identity_id = Filter(
            help_text="filter the results by the identity that created the Profile. Use a full STIX identity ID, e.g. `identity--de9fb9bd-7895-4b23-aa03-49250d9263c9`"
        )

    def get_queryset(self):
        return Profile.objects


class txt2stixView(
    mixins.RetrieveModelMixin, mixins.ListModelMixin, viewsets.GenericViewSet
):
    serializer_class = Txt2stixExtractorSerializer
    lookup_url_kwarg = "id"

    def get_queryset(self):
        return None

    @classmethod
    def all_extractors(cls, types):
        return Txt2stixExtractorSerializer.all_extractors(tuple(types))

    def get_all(self):
        raise NotImplementedError("not implemented")

    def list(self, request, *args, **kwargs):
        page = self.paginate_queryset(list(self.get_all().values()))
        return self.get_paginated_response(page)

    def retrieve(self, request, *args, **kwargs):
        items = self.get_all()
        id_ = self.kwargs.get(self.lookup_url_kwarg)
        print(id_, self.lookup_url_kwarg, self.kwargs)
        item = items.get(id_)
        if not item:
            return response.Response(
                dict(message="item not found", code=404), status=404
            )
        return response.Response(item)


@extend_schema_view(
    list=extend_schema(
        summary="Search Extractors",
        description=textwrap.dedent(
            """
            Extractors are what extract the data from the text which is then converted into STIX objects.

            For more information see [txt2stix](https://github.com/muchdogesec/txt2stix/).
            """
        ),
        responses={400: DEFAULT_400_ERROR, 200: Txt2stixExtractorSerializer},
    ),
    retrieve=extend_schema(
        summary="Get an extractor",
        description=textwrap.dedent(
            """
            Get a specific Extractor.
            """
        ),
        responses={
            400: DEFAULT_400_ERROR,
            404: DEFAULT_404_ERROR,
            200: Txt2stixExtractorSerializer,
        },
    ),
)
class ExtractorsView(txt2stixView):
    openapi_tags = ["Extractors"]
    lookup_url_kwarg = "extractor_id"
    openapi_path_params = [
        OpenApiParameter(
            lookup_url_kwarg,
            location=OpenApiParameter.PATH,
            type=OpenApiTypes.STR,
            description="The `id` of the Extractor.",
        )
    ]
    pagination_class = Pagination("extractors")
    filter_backends = [DjangoFilterBackend]

    class filterset_class(FilterSet):
        type = Filter(
            choices=[(extractor, extractor) for extractor in EXTRACTOR_TYPES],
            help_text="Filter Extractors by their `type`",
        )
        name = Filter(
            help_text="Filter extractors by `name`. Is wildcard search so `ip` will return `ipv4`, `ipv6`, etc.)"
        )
        web_app = BooleanFilter(
            help_text="filters on `dogesec_web` property in txt2stix filter.\nuse case is, web app can set this to true to only show extractors allowed in web app"
        )

    def get_all(self):
        types = EXTRACTOR_TYPES
        if type := self.request.GET.get("type"):
            types = type.split(",")

        extractors = self.all_extractors(types)

        if name := self.request.GET.get("name", "").lower():
            extractors = {
                slug: extractor
                for slug, extractor in extractors.items()
                if name in extractor["name"].lower()
            }

        webapp_filter = NullBooleanField.to_python(
            ..., self.request.GET.get("web_app", "")
        )
        if webapp_filter != None:
            extractors = {
                slug: extractor
                for slug, extractor in extractors.items()
                if extractor.get("dogesec_web") == webapp_filter
            }
        return extractors



# Generated by Django 5.2.4 on 2025-10-05 10:41

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0008_profile_ai_create_attack_navigator_layer'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='include_embedded_relationships_attributes',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=128), default=list, size=None),
        ),
    ]



"""
URL configuration for dogesec_commons project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.contrib import admin
from django.http import JsonResponse
from django.urls import include, path
from rest_framework import routers
from dogesec_commons.stixifier.views import ExtractorsView, ProfileView
from drf_spectacular.views import SpectacularAPIView, SpectacularSwaggerView
from dogesec_commons.objects import views as arango_views

router = routers.SimpleRouter(use_regex_path=False)

router.register("profiles", ProfileView, "profile-view")
# txt2stix views
router.register("extractors", ExtractorsView, "extractors-view")

## objects
regex_router = routers.SimpleRouter(use_regex_path=True)
regex_router.register("", arango_views.ObjectsWithReportsView, "object-view-orig")
regex_router.register("smos", arango_views.SMOView, "object-view-smo")
regex_router.register("scos", arango_views.SCOView, "object-view-sco")
regex_router.register("sros", arango_views.SROView, "object-view-sro")
regex_router.register("sdos", arango_views.SDOView, "object-view-sdo")
urlpatterns = [
    path(f"api/", include(router.urls)),
    path(f"objects/", include(regex_router.urls)),
    path("admin/", admin.site.urls),
    # YOUR PATTERNS
    path("api/schema/", SpectacularAPIView.as_view(), name="schema"),
    # Optional UI:
    path(
        "api/schema/swagger-ui/",
        SpectacularSwaggerView.as_view(url_name="schema"),
        name="swagger-ui",
    ),
]


def handler404(*args, **kwargs):
    return JsonResponse(dict(code=404, message="non-existent page"), status=404)


def handler500(*args, **kwargs):
    return JsonResponse(dict(code=500, message="internal server error"), status=500)



# Generated by Django 5.1.5 on 2025-06-24 12:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0004_profile_identity_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='generate_pdf',
            field=models.BooleanField(default=False),
        ),
    ]



import uuid
import pytest
from dogesec_commons.stixifier.serializers import (
    validate_model,
    validate_stix_id,
    ProfileSerializer,
)
from rest_framework.validators import ValidationError


def test_validate_model():
    assert validate_model("openai") == "openai"
    assert validate_model("") == None
    with pytest.raises(ValidationError):
        validate_model("random:10s")


def test_validate_stix_id():
    id = str(uuid.uuid4())
    assert validate_stix_id("report--" + id, "report") == "report--" + id
    assert validate_stix_id("indicator--" + id, "indicator") == "indicator--" + id
    with pytest.raises(ValidationError):
        validate_stix_id("indicator--" + id, "report")
    with pytest.raises(ValidationError):
        validate_stix_id("indicator--bad-id", "indicator")


@pytest.mark.django_db
def test_include_embedded_relationships_attributes():
    s = ProfileSerializer(
        data={
            "extractions": ["pattern_domain_name_only"],
            "defang": False,
            "name": "my-name",
        }
    )
    s.is_valid(raise_exception=True)

    with pytest.raises(ValidationError, match="value must end with _ref or _refs"):
        s = ProfileSerializer(
            data={
                "extractions": ["pattern_domain_name_only"],
                "defang": False,
                "name": "my-name",
                "include_embedded_relationships_attributes": ["abcde_ref", "acd"],
            }
        )
        s.is_valid(raise_exception=True)
    s = ProfileSerializer(
        data={
            "extractions": ["pattern_domain_name_only"],
            "defang": False,
            "name": "my-name",
            "include_embedded_relationships_attributes": ["abcde_ref"],
        }
    )
    s.is_valid(raise_exception=True)



from drf_spectacular.utils import OpenApiResponse, OpenApiExample
from .serializers import CommonErrorSerializer


HTTP404_EXAMPLE = OpenApiExample("http-404", {"message": "resource not found", "code": 404})
HTTP400_EXAMPLE = OpenApiExample("http-400", {"message": "request not understood", "code": 400})

WEBSERVER_404_RESPONSE = OpenApiResponse(CommonErrorSerializer, description="webserver's HTML 404 page", examples=[OpenApiExample('404-page', {"code": 404, "message": "non-existent page"})])
WEBSERVER_500_RESPONSE = OpenApiResponse(CommonErrorSerializer, description="webserver's HTML 500 page", examples=[OpenApiExample('500-page', {"code": 500, "message": "internal server error"})])


DEFAULT_400_RESPONSE = OpenApiResponse(
    CommonErrorSerializer,
    "The server did not understand the request",
    [
        HTTP400_EXAMPLE
    ],
)


DEFAULT_404_RESPONSE = OpenApiResponse(
    CommonErrorSerializer,
    "Resource not found",
    [
        HTTP404_EXAMPLE
    ],
)



# Generated by Django 5.1.5 on 2025-07-07 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0005_profile_generate_pdf'),
    ]

    operations = [
        migrations.AlterField(
            model_name='profile',
            name='ai_create_attack_flow',
            field=models.BooleanField(default=False),
        ),
    ]



from django.apps import AppConfig

class ArangoObjectsViewApp(AppConfig):
    name = 'dogesec_commons.objects'
    label = 'dogesec_arango_objects_views'

    def ready(self) -> None:
        from .db_view_creator import startup_func
        startup_func()
        return super().ready()


import contextlib
from django.conf import settings
from rest_framework import pagination, response
from rest_framework import response
from rest_framework.exceptions import NotFound
from django.core.paginator import Page as DjangoPage, InvalidPage


class Pagination(pagination.PageNumberPagination):
    max_page_size = settings.MAXIMUM_PAGE_SIZE
    page_size = settings.DEFAULT_PAGE_SIZE
    page_size_query_param = 'page_size'
    def __init__(self, results_key) -> None:
        self.results_key = results_key
        super().__init__()

    def paginate_queryset(self, queryset, request, view=None):
        with contextlib.suppress(NotFound):
            return super().paginate_queryset(queryset, request, view)
        self.page = DjangoPage([], -1, self)
        return []
    
    def paginate_queryset(self, queryset, request, view=None):
        """
        Paginate a queryset if required, either returning a
        page object, or `None` if pagination is not configured for this view.
        """
        self.request = request
        page_size = self.get_page_size(request)
        if not page_size:
            return None

        paginator = self.django_paginator_class(queryset, page_size)
        page_number = self.get_page_number(request, paginator)

        try:
            self.page = paginator.page(page_number)
        except InvalidPage as exc:
            if isinstance(page_number, str):
                page_number = int(page_number) if page_number.isdigit() else -1
            self.page = DjangoPage([], page_number, paginator)

        return list(self.page)
    
    def get_paginated_response(self, data):
        
        return response.Response({
            'page_size': self.get_page_size(self.request),
            'page_number': self.page.number,
            'page_results_count': len(self.page),
            'total_results_count': self.page.paginator.count,
            self.results_key: data,
        })

    def get_paginated_response_schema(self, schema):
        return {
            'type': 'object',
            'required': ['total_results_count', self.results_key],
            'properties': {
                'page_size': {
                    'type': 'integer',
                    'example': self.max_page_size,
                },
                'page_number': {
                    'type': 'integer',
                    'example': 3,
                },
                'page_results_count': {
                    'type': 'integer',
                    'example': self.max_page_size,
                },
                'total_results_count': {
                    'type': 'integer',
                    'example': 3,
                },
                self.results_key: schema,
            },
        }

    def __call__(self, *args, **kwargs):
        return self.__class__(results_key=self.results_key)



import uuid
from django.test import TransactionTestCase
from django.urls import include, path
from rest_framework.test import APIRequestFactory
from unittest.mock import patch
from rest_framework.test import URLPatternsTestCase
from rest_framework import routers

from dogesec_commons.stixifier.models import Profile
from dogesec_commons.stixifier.views import ExtractorsView, ProfileView

factory = APIRequestFactory()


def get_profile_data():
    return {
        "name": "test-profile",
        "extractions": ["pattern_host_name"],
        "extract_text_from_image": False,
        "defang": True,
        "relationship_mode": "standard",
        "ai_settings_relationships": None,
        "ai_settings_extractions": [],
        "ai_content_check_provider": None,
        "ai_create_attack_flow": False,
    }


class ProfileViewTest(TransactionTestCase, URLPatternsTestCase):
    router = routers.SimpleRouter()
    router.register("", ProfileView, "profiles-view")
    urlpatterns = [
        path("profiles/", include(router.urls)),
    ]
    stix_id = "stix-object--" + str(uuid.uuid4())

    def test_create_profile(self):
        profile_data = get_profile_data()
        response = self.client.post(
            "/profiles/", profile_data, content_type="application/json"
        )
        assert response.status_code == 201, response.data
        assert "id" in response.data
        for k in [
            "name",
            "extractions",
            "extract_text_from_image",
            "relationship_mode",
            "defang",
        ]:
            assert profile_data[k] == response.data[k]

    def test_list_profiles(self):
        p = Profile.objects.create(**get_profile_data())
        response = self.client.get("/profiles/")
        assert response.status_code == 200
        profiles = response.data["profiles"]
        assert isinstance(profiles, list)
        assert str(p.id) == profiles[0]["id"]

    def test_retrieve_profiles(self):
        p = Profile.objects.create(**get_profile_data())
        response = self.client.get(f"/profiles/{p.id}/")
        assert response.status_code == 200
        profile = response.data
        assert isinstance(profile, dict)
        assert str(p.id) == profile["id"]

    def test_delete_profiles(self):
        p = Profile.objects.create(**get_profile_data())
        response = self.client.delete(f"/profiles/{p.id}/")
        assert response.status_code == 204

        response = self.client.get(f"/profiles/{p.id}/")
        assert response.status_code == 404, "should already be deleted"


class ExtractorsViewTest(URLPatternsTestCase):
    router = routers.SimpleRouter()
    router.register("", ExtractorsView, "extractors-view")
    urlpatterns = [
        path("extractors/", include(router.urls)),
    ]

    def list_extractor_tester(self, filters):
        response = self.client.get(f"/extractors/", query_params=filters)
        assert response.status_code == 200
        extractors = response.data["extractors"]

        if type := filters.get("type"):
            assert {ex["type"] for ex in extractors} == {type}

        if (web_app := filters.get("web_app", None)) != None:
            assert {ex.get("dogesec_web") for ex in extractors} == {web_app}

        if name := filters.get("name"):
            for ex in extractors:
                assert name.lower() in ex.get("name", "").lower()

    def test_list_extractors(self):
        filters = [
            dict(name="ipv4"),
            dict(name="ipv4", web_app=True),
            dict(),
            dict(type="ai"),
            dict(type="pattern"),
            dict(type="lookup"),
        ]
        for filter in filters:
            with self.subTest(f"test_filters: {filter}", filters=filters):
                self.list_extractor_tester(filter)

    def test_retrieve_extractor(self):
        with patch.object(ExtractorsView, "get_all") as mock_get_all:
            extractor_id = "mocked_id"
            mocked_extractor = {"name": "this should be the response"}
            mock_get_all.return_value = {extractor_id: mocked_extractor}
            response = self.client.get(f"/extractors/{extractor_id}/")
            assert response.status_code == 200
            extractors = response.data == mocked_extractor



import time
from urllib.parse import urlencode
import schemathesis
import pytest
from schemathesis.core.transport import Response as SchemathesisResponse
from dogesec_commons.wsgi import application as wsgi_app
from rest_framework.response import Response as DRFResponse
from schemathesis.specs.openapi.checks import (
    negative_data_rejection,
    positive_data_acceptance,
)
from schemathesis.config import GenerationConfig

schema = schemathesis.openapi.from_wsgi(
    "/api/schema/?format=json",
    wsgi_app,
)
schema.config.base_url = "http://localhost:8005/"
schema.config.generation = GenerationConfig(allow_x00=False)


@pytest.fixture(autouse=True)
def override_transport(monkeypatch, client):
    from schemathesis.transport.wsgi import WSGI_TRANSPORT, WSGITransport

    class Transport(WSGITransport):
        def __init__(self):
            super().__init__()
            self._copy_serializers_from(WSGI_TRANSPORT)

        @staticmethod
        def case_as_request(case):
            from schemathesis.transport.requests import REQUESTS_TRANSPORT
            import requests

            r_dict = REQUESTS_TRANSPORT.serialize_case(
                case,
                base_url=case.operation.base_url,
            )
            return requests.Request(**r_dict).prepare()

        def send(self, case: schemathesis.Case, *args, **kwargs):
            t = time.time()
            case.headers.pop("Authorization", "")
            serialized_request = WSGI_TRANSPORT.serialize_case(case)
            serialized_request.update(
                QUERY_STRING=urlencode(serialized_request["query_string"])
            )
            response: DRFResponse = client.generic(**serialized_request)
            elapsed = time.time() - t
            return SchemathesisResponse(
                response.status_code,
                headers={k: [v] for k, v in response.headers.items()},
                content=response.content,
                request=self.case_as_request(case),
                elapsed=elapsed,
                verify=True,
            )

    ## patch transport.get
    from schemathesis import transport

    monkeypatch.setattr(transport, "get", lambda _: Transport())


@pytest.mark.django_db(transaction=True)
@schema.parametrize()
def test_api(case: schemathesis.Case):
    case.call_and_validate(
        excluded_checks=[negative_data_rejection, positive_data_acceptance]
    )



import io
import uuid
import pytest
import json
from pathlib import Path
from tempfile import NamedTemporaryFile
from unittest.mock import patch, MagicMock
from dogesec_commons.stixifier.models import Profile, RelationshipMode

from txt2stix import txt2stixBundler

from dogesec_commons.stixifier.stixifier import (
    StixifyProcessor,
    ReportProperties,
)


@pytest.fixture
def fake_file():
    tmp_file = NamedTemporaryFile(delete=False, suffix=".html")
    tmp_file.write(b"<html>Example content</html>")
    tmp_file.seek(0)
    tmp_file.name = "example.html"
    return tmp_file


@pytest.fixture
def fake_profile(db):
    return Profile.objects.create(
        name="test-profile",
        extract_text_from_image=False,
        defang=False,
        ignore_image_refs=False,
        ignore_link_refs=False,
        extractions=[],
        ai_content_check_provider=None,
        ai_create_attack_flow=False,
        ai_settings_extractions=[],
        ai_settings_relationships=None,
        relationship_mode=RelationshipMode.STANDARD,
        ignore_extraction_boundary=False,
        ignore_embedded_relationships=False,
        ignore_embedded_relationships_smo=False,
        ignore_embedded_relationships_sro=False,
        include_embedded_relationships_attributes=["object_refs", "created_by_ref"],
    )


def test_init(fake_file, fake_profile):
    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4())
    assert processor.filename.exists()
    assert processor.tmpdir.exists()
    assert processor.profile == fake_profile


def test_file2txt(fake_file, fake_profile):
    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4())

    with patch(
        "dogesec_commons.stixifier.stixifier.get_parser_class"
    ) as mock_get_parser:
        mock_converter = MagicMock()
        mock_converter.convert.return_value = "converted text"
        mock_converter.images = {}
        mock_get_parser.return_value.return_value = mock_converter

        processor.file2txt()
        assert processor.output_md == "converted text"
        assert processor.md_file.exists()


def test_setup(fake_file, fake_profile):
    original_extra = dict(extra_1=1)
    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4())
    processor.extra_data = original_extra
    setup_extra = dict(setup_extra=2)
    report_prop = ReportProperties(name="Test Report")
    processor.setup(report_prop, setup_extra)
    assert report_prop == processor.report_prop
    assert processor.extra_data == {
        **original_extra,
        **setup_extra,
    }, "setup should update extra_data"


def test_txt2stix(fake_file, fake_profile, settings):
    fake_profile.extractions = ["test-extractor"]
    fake_profile.ai_settings_extractions = ["some-ai-model"]
    fake_profile.ai_content_check_provider = None
    fake_profile.ai_create_attack_flow = True
    fake_profile.ignore_image_refs = True
    fake_profile.ignore_link_refs = True
    fake_profile.ignore_extraction_boundary = True
    fake_profile.ai_settings_relationships = "relationship-model"
    fake_profile.relationship_mode = "full"
    fake_profile.ai_extract_if_no_incidence = False
    fake_profile.save()

    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4())
    processor.output_md = "This is a markdown"
    processor.report_id = "fake-id"

    report_props = ReportProperties(
        name="Test Report",
        identity={"type": "identity", "name": "Org"},
        tlp_level="green",
        confidence=85,
        labels=["test"],
        created="2024-01-01T00:00:00Z",
    )
    processor.setup(report_props)

    with (
        patch(
            "dogesec_commons.stixifier.stixifier.all_extractors", return_value={}
        ) as mock_extractors,
        patch(
            "dogesec_commons.stixifier.stixifier.txt2stixBundler"
        ) as mock_bundler_cls,
        patch(
            "dogesec_commons.stixifier.stixifier.txt2stix.txt2stix.run_txt2stix"
        ) as mock_run,
        patch(
            "dogesec_commons.stixifier.stixifier.txt2stix.utils.remove_links",
            return_value="cleaned text",
        ),
        patch(
            "dogesec_commons.stixifier.stixifier.txt2stix.txt2stix.parse_model",
            side_effect=lambda x: f"parsed({x})",
        ) as mock_parse_model,
    ):
        mock_bundler = MagicMock()
        mock_bundler.report.id = "report--abc"
        mock_bundler_cls.return_value = mock_bundler
        mock_run.return_value.content_check = MagicMock()

        result = processor.txt2stix()

        assert result == mock_bundler

        # Validate bundler was instantiated with expected arguments
        mock_bundler_cls.assert_called_once_with(
            "Test Report",
            identity={"type": "identity", "name": "Org"},
            tlp_level="green",
            confidence=85,
            labels=["test"],
            description="This is a markdown",
            extractors={},
            report_id="fake-id",
            created="2024-01-01T00:00:00Z",
        )

        # Validate remove_links was called with correct arguments
        txt = processor.output_md
        assert txt == "This is a markdown"
        mock_parse_model.assert_any_call("some-ai-model")
        mock_parse_model.assert_any_call("relationship-model")

        # Validate run_txt2stix was called correctly
        mock_run.assert_called_once_with(
            mock_bundler,
            "cleaned text",
            {},
            ai_content_check_provider=None,
            ai_create_attack_flow=True,
            ai_create_attack_navigator_layer=False,
            input_token_limit=settings.INPUT_TOKEN_LIMIT,
            ai_settings_extractions=["parsed(some-ai-model)"],
            ai_settings_relationships="parsed(relationship-model)",
            relationship_mode="full",
            ignore_extraction_boundary=True,
            ai_extract_if_no_incidence=False,
        )

        assert processor.txt2stix_data == mock_run.return_value
        assert processor.incident == mock_run.return_value.content_check
        assert processor.summary == mock_run.return_value.content_check.summary


def test_write_bundle(fake_file, fake_profile):
    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4(), report_id="abc")
    mock_bundler = MagicMock()
    mock_bundler.to_json.return_value = json.dumps({"type": "bundle", "id": "example"})
    processor.write_bundle(mock_bundler)
    assert processor.bundle_file.exists()


def test_upload_to_arango(fake_file, fake_profile):
    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4(), report_id="abc")
    processor.bundle_file = Path(processor.tmpdir / "bundle_abc.json")
    processor.bundle_file.write_text("{}")

    with (
        patch("dogesec_commons.stixifier.stixifier.Stix2Arango") as mock_s2a,
        patch(
            "dogesec_commons.stixifier.stixifier.db_view_creator.link_one_collection"
        ) as mock_link,
    ):
        mock_instance = mock_s2a.return_value
        processor.upload_to_arango()
        mock_instance.run.assert_called()
        assert mock_link.call_count == 2
        mock_s2a.assert_called_once_with(
            file=str(processor.bundle_file),
            database="test_dogesec_commons",
            collection="stixify",
            stix2arango_note="stixifier-report--abc",
            ignore_embedded_relationships=False,
            ignore_embedded_relationships_smo=False,
            ignore_embedded_relationships_sro=False,
            include_embedded_relationships_attributes=["object_refs", "created_by_ref"],
            host_url=mock_s2a.call_args[1]['host_url'],
            username=mock_s2a.call_args[1]['username'],
            password=mock_s2a.call_args[1]['password'],
        )


def test_process(fake_file, fake_profile):
    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4(), report_id="abc")
    with (
        patch.object(StixifyProcessor, "file2txt") as mock_f2t,
        patch.object(StixifyProcessor, "txt2stix") as mock_t2s,
        patch.object(StixifyProcessor, "write_bundle"),
        patch.object(StixifyProcessor, "upload_to_arango") as mock_upload_to_arango,
    ):
        assert processor.process() == mock_t2s.return_value.report.id
        mock_f2t.assert_called_once()
        mock_t2s.assert_called_once()
        mock_upload_to_arango.assert_called_once()


def test_write_bundle(fake_file, fake_profile):
    processor = StixifyProcessor(fake_file, fake_profile, uuid.uuid4(), report_id="abc")
    bundler = MagicMock()
    bundle_dict = {"some-data": [], "other-data": "2"}
    bundler.to_json.return_value = json.dumps(bundle_dict)
    processor.write_bundle(bundler)
    assert processor.bundle_file.exists()
    assert json.loads(processor.bundle_file.read_text()) == bundle_dict



from django.conf import settings
from rest_framework import pagination, response
from rest_framework.filters import OrderingFilter
from django.utils.encoding import force_str
from rest_framework import response

class Ordering(OrderingFilter):
    ordering_param = "sort"

    def get_ordering(self, request, queryset, view):
        params = request.query_params.get(self.ordering_param)
        ordering_mapping = self.get_ordering_mapping(queryset, view)
        if params:
            fields = [ordering_mapping.get(param.strip()) for param in params.split(',') if param.strip() in ordering_mapping]
            ordering = self.remove_invalid_fields(queryset, fields, view, request)
            if ordering:
                return ordering
        return self.get_default_ordering(view)

    def get_ordering_mapping(self, queryset, view):
        valid_fields = self.get_valid_fields(queryset, view)
        mapping = {}
        for k, v in valid_fields:
            mapping[f"{k}_descending"] = f"-{v}"
            mapping[f"{k}_ascending"]  = v
        return mapping
    

    def get_schema_operation_parameters(self, view):
        return [
            {
                'name': self.ordering_param,
                'required': False,
                'in': 'query',
                'description': force_str(self.ordering_description),
                'schema': {
                    'type': 'string',
                    'enum': list(self.get_ordering_mapping(None, view).keys())
                },
            },
        ]
    
    def get_default_ordering(self, view):
        ordering = getattr(view, 'ordering', None)
        if isinstance(ordering, str):
            return (self.get_ordering_mapping(None, view).get(ordering),)
        return None


import txt2stix
from django.conf import settings
from django.db import models
from django.contrib.postgres.fields import ArrayField
import uuid
from functools import partial
import txt2stix.common
import txt2stix, txt2stix.extractions
from django.core.exceptions import ValidationError


class RelationshipMode(models.TextChoices):
    AI = "ai", "AI Relationship"
    STANDARD = "standard", "Standard Relationship"


def validate_extractor(types, name):
    extractors = txt2stix.extractions.parse_extraction_config(
        txt2stix.txt2stix.INCLUDES_PATH
    ).values()
    for extractor in extractors:
        if name == extractor.slug and extractor.type in types:
            return True
    raise ValidationError(f"{name} does not exist", 400)


class Profile(models.Model):
    id = models.UUIDField(primary_key=True)
    created = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=250, unique=True)
    identity_id = models.CharField(max_length=46, null=True, default=None)
    extractions = ArrayField(base_field=models.CharField(max_length=256))
    relationship_mode = models.CharField(
        choices=RelationshipMode.choices,
        max_length=20,
        default=RelationshipMode.STANDARD,
    )
    extract_text_from_image = models.BooleanField(default=False)
    defang = models.BooleanField()
    generate_pdf = models.BooleanField(default=False)
    ai_settings_relationships = models.CharField(max_length=256, blank=False, null=True)
    ai_settings_extractions = ArrayField(
        base_field=models.CharField(max_length=256), default=list
    )
    ai_content_check_provider = models.CharField(
        default=None, null=True, blank=False, max_length=256
    )
    ai_extract_if_no_incidence = models.BooleanField(default=True)
    ai_create_attack_flow = models.BooleanField(default=False)
    ai_create_attack_navigator_layer = models.BooleanField(default=False)
    ignore_image_refs = models.BooleanField(default=True)
    ignore_link_refs = models.BooleanField(default=True)
    ignore_extraction_boundary = models.BooleanField(default=False)

    #############
    ignore_embedded_relationships_sro = models.BooleanField(default=True)
    ignore_embedded_relationships_smo = models.BooleanField(default=True)
    ignore_embedded_relationships = models.BooleanField(default=False)
    include_embedded_relationships_attributes = ArrayField(base_field=models.CharField(max_length=128), default=list)

    def save(self, *args, **kwargs) -> None:
        if not self.id:
            name = self.name
            if self.identity_id:
                name = f"{self.name}+{self.identity_id}"
            self.id = uuid.uuid5(settings.STIXIFIER_NAMESPACE, name)
        return super().save(*args, **kwargs)



# Generated by Django 5.2.4 on 2025-07-14 15:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0007_remove_profile_ai_summary_provider_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='ai_create_attack_navigator_layer',
            field=models.BooleanField(default=False),
        ),
    ]



from typing import List
from drf_spectacular.openapi import AutoSchema
from drf_spectacular.plumbing import ComponentRegistry
from drf_spectacular.utils import _SchemaType, OpenApiResponse, OpenApiExample
from drf_spectacular.types import OpenApiTypes
import uritemplate

from dogesec_commons.utils import schemas
from .serializers import CommonErrorSerializer

from drf_spectacular.contrib.django_filters import DjangoFilterExtension, get_view_model
class OverrideDjangoFilterExtension(DjangoFilterExtension):
    priority = 10
    def get_schema_operation_parameters(self, auto_schema: AutoSchema, *args, **kwargs):
        model = get_view_model(auto_schema.view)
        if not model:
            return self.override(auto_schema, *args, **kwargs)
        return super().get_schema_operation_parameters(auto_schema, *args, **kwargs)
    
    def override(self, autoschema, *args, **kwargs):
        result = []
        filterset_class = self.target.get_filterset_class(autoschema.view)
        if not filterset_class:
            return self.target.get_schema_operation_parameters(autoschema.view, *args, **kwargs)
        for field_name, filter_field in filterset_class.base_filters.items():
            result += self.resolve_filter_field(
                autoschema, None, filterset_class, field_name, filter_field
            )
        return result


class CustomAutoSchema(AutoSchema):
    default_responses = {
            '404': (schemas.WEBSERVER_404_RESPONSE, ["application/json"]),
    }
    def get_tags(self) -> List[str]:
        if hasattr(self.view, "openapi_tags"):
            return self.view.openapi_tags
        return super().get_tags()

    
    def get_override_parameters(self):
        params = super().get_override_parameters()
        path_variables = uritemplate.variables(self.path)
        for param in getattr(self.view, 'openapi_path_params', []):
            if param.name in path_variables:
                params.append(param)
        return params
    
    def _map_serializer_field(self, field, direction, bypass_extensions=False):
        if getattr(field, 'internal_serializer', None):
            return super()._map_serializer_field(field.internal_serializer, direction, bypass_extensions)
        return super()._map_serializer_field(field, direction, bypass_extensions)


    def _map_serializer(self, serializer, direction, bypass_extensions=False):
        if getattr(serializer, "get_schema", None):
            return serializer.get_schema()
        return super()._map_serializer(serializer, direction, bypass_extensions)
    

    def get_operation(self, *args, **kwargs):
        operation = super().get_operation(*args, **kwargs)
        if operation:
            self.add_default_pages(operation)
        return operation

    def add_default_pages(self, operation):
        """
        modify responses to include 404 error for when path parameters don't match specified path. e.g if integer passed instead of a uuid param
        """
        responses = operation['responses']

        default_responses = {
            code: self._get_response_for_code(schema, code, content_type) for code, (schema, content_type) in self.default_responses.items()
        }
        for code, content_response in default_responses.items():
            if code not in responses:
                responses[code] = content_response
        return operation
    
    def _is_list_view(self, serializer=None) -> bool:
        if getattr(self.view, 'action', None) == 'list' and getattr(self.view, 'skip_list_view', False):
            """
            view.skip_list_view is used for checking if many should be used or not on list() action
            """
            return False
        return super()._is_list_view(serializer)


from django.conf import settings

MAXIMUM_PAGE_SIZE = getattr(settings, 'MAXIMUM_PAGE_SIZE', 200)
DEFAULT_PAGE_SIZE = getattr(settings, 'DEFAULT_PAGE_SIZE', 50)

DB = settings.ARANGODB_DATABASE
DB_NAME = f"{DB}_database"
ARANGODB_DATABASE_VIEW = getattr(settings, "ARANGODB_DATABASE_VIEW", f"{DB}_view")